#include "MESH_SelectionView.h"
#include "MESH_VerticesView.h"
#include "MESH_EdgesView.h"
#include "MESH_FacesView.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

typedef enum {NIL, VERTEX, EDGE, FACE} Entity;

SelectionView::SelectionView()
: QView("selection"), _verticesView(0), _edgesView(0), _facesView(0),
 _mesh(0), _face(0), _edge(0), _vertex(0), _transform(SFMatrix34f::identity)
{
}

SelectionView::~SelectionView()
{
}
  
void SelectionView::receiveNotification()
{
  _mesh = 0;
  _face = 0;
  _edge = 0;
  _vertex = 0;
  _transform = SFMatrix34f::identity;
}

void SelectionView::display()
{
  glDisable(GL_LIGHTING);
  glPushMatrix();
  glMultMatrixf(_transform.toFloat16());

  glEnable(GL_POLYGON_OFFSET_FILL);
#ifdef GL_POLYGON_OFFSET_LINES
  glEnable(GL_POLYGON_OFFSET_LINES);
#endif

  glColor3f(0.9f, 0.3f, 0.3f);

  if (_face != 0)  
  {
    const Mesh::MFEdge &edges = _face->getEdges();
    glBegin(GL_POLYGON);
    for (Mesh::MFEdge::const_iterator e = edges.begin(), end = edges.end(); e != end; ++e)
    {
      SFPoint3f point = (*e)->getFromVertex()->data().getPoint();
      glVertex3fv(point.f_data());
    }
    glEnd();
  }

  if (_edge != 0)  
  {
    glLineWidth(5.0);
    glBegin(GL_LINES);
    glVertex3fv(_edge->getFromVertex()->data().getPoint().f_data());
    glVertex3fv(_edge->getToVertex()->data().getPoint().f_data());
    glEnd();
  }
  
  if (_vertex != 0)  
  {
    glColor3f(0.9f, 0.3f, 0.3f);
    glPointSize(5.0);
    glBegin(GL_POINTS);
    glVertex3fv(_vertex->data().getPoint().f_data());
    glEnd();
  }

  glPopMatrix();

  glDisable(GL_POLYGON_OFFSET_FILL);  
#ifdef GL_POLYGON_OFFSET_LINES
  glDisable(GL_POLYGON_OFFSET_LINES);
#endif
}

void SelectionView::select()
{
  Entity entity = NIL;
  unsigned int value = numeric_limits<unsigned int>::max();
  unsigned int currentValue = 0;
  
  if ((_verticesView != 0) 
   && (_verticesView->isVisible()) 
   && (_verticesView->getSelectedVertex() != 0))
  { 
    currentValue = _verticesView->getValue();
    if (currentValue != numeric_limits<unsigned int>::max())
      currentValue -= (1 << 24);
    if (currentValue < value)
    {
      entity = VERTEX;
      value = currentValue;
    }  
  }
  if ((_edgesView != 0) 
   && (_edgesView->isVisible())
   && (_edgesView->getSelectedEdge() != 0)) 
  {
    currentValue = _edgesView->getValue();
    if (currentValue < value)
    {
      entity = EDGE;
      value = currentValue;
    } 
  }
  if ((_facesView != 0)
   && (_facesView->isVisible())
   && (_facesView->getSelectedFace() != 0))
  {
    currentValue = _facesView->getValue();
    if (currentValue < value)
    {
      entity = FACE;
      value = currentValue;
    }  
  }
    
  switch (entity)
  {
    case VERTEX:
      _mesh = _verticesView->getSelectedMesh();
      _transform = _verticesView->getSelectedTransform();
      
      _face = 0;
      _edge = 0;
      _vertex = _verticesView->getSelectedVertex();
      break;
     
    case EDGE:
      _mesh = _edgesView->getSelectedMesh();
      _transform = _edgesView->getSelectedTransform();
      
      _face = 0;
      _edge = _edgesView->getSelectedEdge();
      _vertex = 0;
      break;
    
    case FACE:    
      _mesh = _facesView->getSelectedMesh();
      _transform = _facesView->getSelectedTransform();
      
      _face = _facesView->getSelectedFace();
      _edge = 0;
      _vertex = 0;
      break;
    
    case NIL:
      _mesh = 0;
      _face = 0;
      _edge = 0;
      _vertex = 0;
      _transform = SFMatrix34f::identity;
      break;
  }
  
  emit updated();
}

void SelectionView::show()
{
  QView::show();
  //_vBox->show();
}

void SelectionView::hide()
{
  QView::hide();
  //_vBox->hide();
}

void SelectionView::setVerticesView(VerticesView *verticesView)
{
  _verticesView = verticesView;
}

void SelectionView::setEdgesView(EdgesView *edgesView)
{
  _edgesView = edgesView;
}

void SelectionView::setFacesView(FacesView *facesView)
{
  _facesView = facesView;
}

}
}
